@@ -193,6 +193,10 @@ cdebconf_newt_create_window(const int width, const int height, const char *title
     };
     char *buf = NULL;
     int i = -1;
+    int brl = 0;
+
+    if (access("/var/run/brltty.pid", O_RDONLY) == 0)
+        brl = 1;
 
     if (priority != NULL) {
         for (i = 0; sigils[i][0] != NULL; i++) {
@@ -203,12 +207,17 @@ cdebconf_newt_create_window(const int width, const int height, const char *title
             if (asprintf(&buf, "[%s] %s", sigils[i][1], title) == -1)
                 buf = NULL;
     }
-    if (buf != NULL) {
-        newtCenteredWindow(width, height, buf);
-        free(buf);
-    } else {
+
+    if (buf != NULL)
+        title = buf;
+
+    if (brl)
+        newtOpenWindow(1, 1, width, height, title);
+    else
         newtCenteredWindow(width, height, title);
-    }
+
+    if (buf != NULL)
+        free(buf);
 }
 
 int
