@@ -60,6 +60,9 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <readline/readline.h>
+#include <readline/history.h>
+
 struct frontend_data {
 	char *previous_title;
 };
@@ -198,11 +201,15 @@ static unsigned text_handler_displaydesc(struct frontend *obj, unsigned printed,
 	return printed;
 }
 
-static void
-get_answer(char *answer, int size)
+static char *
+get_answer(const char *prompt)
 {
-	fgets(answer, size, stdin);
-	CHOMP(answer);
+	char *answer;
+	printf("%s\n", prompt);
+	answer = readline("");
+	if (!answer)
+		answer = strdup("");
+	return answer;
 }
 
 static unsigned
@@ -458,10 +465,11 @@ printlist (struct frontend *obj, unsigned max_lines, unsigned start, struct ques
  */
 static int text_handler_boolean(struct frontend *obj, unsigned printed, struct question *q)
 {
-	char buf[30];
+	char *buf;
 	int ans = 0;
 	int def = 0;
 	const char *defval;
+	char *prompt;
 
 	defval = question_getvalue(q, "");
 	if (defval)
@@ -472,31 +480,47 @@ static int text_handler_boolean(struct frontend *obj, unsigned printed, struct q
 			def = 2;
 	}
 
+	if (def)
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt-default",
+				"Prompt: '%c' for help, default=%d> "), CHAR_HELP, def);
+	else
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt",
+				"Prompt: '%c' for help> "), CHAR_HELP);
+
+	asprintf(&buf, "%d: %s%s", 2, question_get_text(obj, "debconf/no", "No"), def == 2 ? " [*]" : "");
+	add_history(buf);
+	free(buf);
+	asprintf(&buf, "%d: %s%s", 1, question_get_text(obj, "debconf/yes", "Yes"), def == 1 ? " [*]" : "");
+	add_history(buf);
+	free(buf);
+
 	do {
 		printf("  %d: %s%s", 1, question_get_text(obj, "debconf/yes", "Yes"), (1 == def ? " [*]" : "    "));
 		printf("  %d: %s%s", 2, question_get_text(obj, "debconf/no", "No"), (2 == def ? " [*]" : ""));
 		printf("\n");
-		if (def)
-			printf(question_get_text(obj, "debconf/text-prompt-default",
-					"Prompt: '%c' for help, default=%d> "), CHAR_HELP, def);
-		else
-			printf(question_get_text(obj, "debconf/text-prompt",
-					"Prompt: '%c' for help> "), CHAR_HELP);
-		get_answer(buf, sizeof(buf));
+		buf = get_answer(prompt);
 		if (buf[0] == CHAR_HELP && buf[1] == 0)
 			printed += show_help(obj, q);
 		else if (obj->methods.can_go_back (obj, q) &&
-		         buf[0] == CHAR_GOBACK && buf[1] == 0)
+		         buf[0] == CHAR_GOBACK && buf[1] == 0) {
+			clear_history();
+			free(buf);
+			free(prompt);
 			return DC_GOBACK;
+		}
 		else if (buf[0] == '1')
 			ans = 1;
 		else if (buf[0] == '2')
 			ans = 2;
 		else if (defval != 0 && ISEMPTY(buf))
 			ans = def;
+		free(buf);
 	} while (ans == 0);
 
 	question_setvalue(q, (ans == 1 ? "true" : "false"));
+
+	clear_history();
+	free(prompt);
 	return DC_OK;
 }
 
@@ -591,11 +615,12 @@ static int text_handler_multiselect(struct frontend *obj, unsigned printed, stru
 	struct choices *choices = NULL;
 	char **defaults;
 	char *defval, *cp;
-	char answer[4096] = {0};
+	char *answer = NULL;
 	int i, j, dcount, choice;
 	int ret = DC_OK;
 	unsigned start = 0;
 	bool all;
+	char *prompt;
 
 	choices = choices_get(obj, q);
 	if (choices == NULL)
@@ -627,11 +652,12 @@ static int text_handler_multiselect(struct frontend *obj, unsigned printed, stru
 			strcat(defval, buf);
 		}
 
+	asprintf(&prompt, question_get_text(obj, "debconf/text-prompt-default-string",
+		"Prompt: '%c' for help, default=%s> "), CHAR_HELP, defval);
   DISPLAY:
 	all = printlist (obj, getheight() - printed - 1, start, q, choices);
-	printf(question_get_text(obj, "debconf/text-prompt-default-string", 
-		"Prompt: '%c' for help, default=%s> "), CHAR_HELP, defval);
-	get_answer(answer, sizeof(answer));
+	free(answer);
+	answer = get_answer(prompt);
 	if (answer[0] == CHAR_HELP && answer[1] == 0)
 	{
 		printed = 0;
@@ -696,6 +722,8 @@ static int text_handler_multiselect(struct frontend *obj, unsigned printed, stru
 	question_setvalue(q, answer);
 
   CleanUp_DEFVAL:
+	free(answer);
+	free(prompt);
 	free(defval);
 	choices_delete(choices);
 	
@@ -716,12 +744,13 @@ static int text_handler_multiselect(struct frontend *obj, unsigned printed, stru
 static int text_handler_select(struct frontend *obj, unsigned printed, struct question *q)
 {
 	struct choices *choices = NULL;
-	char answer[128];
+	char *answer;
 	int i, choice, def = -1;
 	const char *defval;
 	int ret = DC_OK;
 	unsigned start = 0;
 	bool all;
+	char *prompt;
 
 	choices = choices_get(obj, q);
 	if (choices == NULL)
@@ -742,19 +771,28 @@ static int text_handler_select(struct frontend *obj, unsigned printed, struct qu
 			}
 	}
 
+	if (def >= 0 && choices->choices_translated[def]) {
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt-default",
+			"Prompt: '%c' for help, default=%d> "),
+				CHAR_HELP, def+1);
+	} else {
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt",
+			"Prompt: '%c' for help> "), CHAR_HELP);
+	}
+
+	for (i = choices->count; i > 0; i--) {
+		asprintf(&answer, "%d: %s%s", i, choices->choices_translated[i-1], i == def+1 ? " [*]" : "");
+		add_history(answer);
+		free(answer);
+	}
+
+	answer = NULL;
 	i = 0;
 	choice = -1;
 	do {
 		all = printlist (obj, getheight() - printed - 1, start, q, choices);
-		if (def >= 0 && choices->choices_translated[def]) {
-			printf(question_get_text(obj, "debconf/text-prompt-default", 
-				"Prompt: '%c' for help, default=%d> "),
-					CHAR_HELP, def+1);
-		} else {
-			printf(question_get_text(obj, "debconf/text-prompt",
-				"Prompt: '%c' for help> "), CHAR_HELP);
-		}
-		get_answer(answer, sizeof(answer));
+		free(answer);
+		answer = get_answer(prompt);
 		if (answer[0] == CHAR_HELP)
 		{
 			printed += show_help(obj, q);
@@ -807,6 +845,9 @@ static int text_handler_select(struct frontend *obj, unsigned printed, struct qu
 	question_setvalue(q, choices->choices[choices->tindex[choice]]);
 
   CleanUp_SELECTED:
+	clear_history();
+	free(answer);
+	free(prompt);
 	choices_delete(choices);
 	
 	return ret;
@@ -823,21 +864,27 @@ static int text_handler_select(struct frontend *obj, unsigned printed, struct qu
  */
 static int text_handler_note(struct frontend *obj, unsigned printed, struct question *q)
 {
-	char buf[100] = {0};
-	printf("%s ", question_get_text(obj, "debconf/cont-prompt",
+	char *buf;
+	char *prompt;
+	asprintf(&prompt, "%s ", question_get_text(obj, "debconf/cont-prompt",
 			       "[Press enter to continue]"));
-	fflush(stdout);
 	while (1)
 	{
-		get_answer(buf, sizeof(buf));
+		buf = get_answer(prompt);
 		if (buf[0] == CHAR_HELP && buf[1] == 0)
 			printed += show_help(obj, q);
 		else if (obj->methods.can_go_back (obj, q) &&
-		         buf[0] == CHAR_GOBACK && buf[1] == 0)
+		         buf[0] == CHAR_GOBACK && buf[1] == 0) {
+			free(buf);
+			free(prompt);
 			return DC_GOBACK;
+		}
 		else
 			break;
+		free(buf);
 	}
+	free(buf);
+	free(prompt);
 	return DC_OK;
 }
 
@@ -906,23 +953,27 @@ static int text_handler_password(struct frontend *obj, unsigned printed, struct
  */
 static int text_handler_string(struct frontend *obj, unsigned printed, struct question *q)
 {
-	char buf[1024] = {0};
+	char *buf;
 	const char *defval = question_getvalue(q, "");
+	char *prompt;
+	if (defval)
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt-default-string", "Prompt: '%c' for help, default=%s> "), CHAR_HELP, defval);
+	else
+		asprintf(&prompt, question_get_text(obj, "debconf/text-prompt", "Prompt: '%c' for help> "), CHAR_HELP);
 	while (1) {
-		if (defval)
-			printf(question_get_text(obj, "debconf/text-prompt-default-string", "Prompt: '%c' for help, default=%s> "), CHAR_HELP, defval);
-		else
-			printf(question_get_text(obj, "debconf/text-prompt", "Prompt: '%c' for help> "), CHAR_HELP);
-		fflush(stdout);
-		get_answer(buf, sizeof(buf));
+		buf = get_answer(prompt);
 		if (buf[0] == CHAR_HELP && buf[1] == 0)
 			printed += show_help(obj, q);
 		else
 			break;
+		free(buf);
 	}
 	if (obj->methods.can_go_back (obj, q) &&
-	    buf[0] == CHAR_GOBACK && buf[1] == 0)
+	    buf[0] == CHAR_GOBACK && buf[1] == 0) {
+		free(buf);
+		free(prompt);
 		return DC_GOBACK;
+	}
 	if (ISEMPTY(buf) && defval == 0)
 		question_setvalue(q, "");
 	else if (ISEMPTY(buf) && defval != 0)
@@ -931,6 +982,8 @@ static int text_handler_string(struct frontend *obj, unsigned printed, struct qu
 		question_setvalue(q, "");
 	else
 		question_setvalue(q, buf);
+	free(buf);
+	free(prompt);
 	return DC_OK;
 }
 
